from imblearn.ensemble import BalancedBaggingClassifier
import numpy as np
from sklearn.tree import DecisionTreeClassifier


from ..monoview.monoview_utils import BaseMonoviewClassifier
from ..utils.base import base_boosting_estimators
from ..utils.hyper_parameter_search import CustomRandint, CustomUniform

classifier_class_name = "IBDT"

class IBDT(BaseMonoviewClassifier, BalancedBaggingClassifier):

    def __init__(self, random_state=None, n_estimators=10,
                 sampling_strategy="auto", base_estimator=DecisionTreeClassifier(),
                 replacement=False, **kwargs):
        super(IBDT, self).__init__(random_state=random_state,
                                               base_estimator=base_estimator,
                                         n_estimators=n_estimators,
                                         sampling_strategy=sampling_strategy,
                                         replacement=replacement)

        self.param_names = ["n_estimators", "sampling_strategy",
                            "base_estimator__max_depth",
                            "base_estimator__criterion",
                            "base_estimator__splitter",]
        self.classed_params = ["base_estimator"]
        self.distribs = [CustomRandint(low=1, high=50),
                         ["auto"],CustomRandint(low=1, high=300),
                         ["gini", "entropy"],
                         ["best", "random"],]
        self.weird_strings=[]

    def fit(self, X, y):
        BalancedBaggingClassifier.fit(self, X, y)
        self.feature_importances_ = np.zeros(X.shape[1])
        for estim in self.estimators_:
            if hasattr(estim['classifier'], 'feature_importances_'):
                self.feature_importances_ += estim['classifier'].feature_importances_
        self.feature_importances_ /= np.sum(self.feature_importances_)
        return self



