import os
import time

import numpy as np
from sklearn.ensemble import AdaBoostClassifier


from .. import metrics
from ..monoview.monoview_utils import BaseMonoviewClassifier, get_accuracy_graph
from summit.multiview_platform.utils.hyper_parameter_search import CustomRandint, CustomUniform
from ..monoview_classifiers.scm import SCM
from ..utils.base import base_boosting_estimators

# Author-Info
__author__ = "Baptiste Bauvin"
__status__ = "Prototype"  # Production, Development, Prototype

classifier_class_name = "SCMboost"


class SCMboost(AdaBoostClassifier, BaseMonoviewClassifier):
    """
    This class is an adaptation of scikit-learn's `AdaBoostClassifier <https://scikit-learn.org/stable/modules/generated/sklearn.ensemble.AdaBoostClassifier.html#sklearn.ensemble.AdaBoostClassifier>`_

    """

    def __init__(self, random_state=None, n_estimators=50,
                 base_estimator=SCM(p=0.49, max_rules=10, model_type="conjunction"),
                  **kwargs):
        if "base_estimator__p" in kwargs:
            base_estimator.p = kwargs["base_estimator__p"]
        if "base_estimator__model_type" in kwargs:
            base_estimator.model_type = kwargs["base_estimator__model_type"]
        AdaBoostClassifier.__init__(self,
                                    random_state=random_state,
                                    n_estimators=n_estimators,
                                    base_estimator=base_estimator,
                                    algorithm="SAMME",)
        self.param_names = ["n_estimators", "base_estimator__p", "base_estimator__model_type"]
        self.classed_params = []
        self.distribs = [CustomRandint(low=1, high=100), CustomUniform(loc=0, state=1), ["conjunction", "disjunction"]]
        self.weird_strings = {}


    # def get_interpretation(self, directory, base_file_name, y_test, feature_ids,
    #                        multi_class=False):  # pragma: no cover
    #     interpretString = ""
    #     return interpretString
