import numpy as np
from sklearn.preprocessing import LabelBinarizer

from imblearn.under_sampling import RandomUnderSampler

from spkm.spkm_wrapper import pairwiseSPKMlikeSklearn
from spkm.kernels_and_gradients import RBFKernel, PolyKernel

from ..multiview.multiview_utils import BaseMultiviewClassifier
from ..utils.hyper_parameter_search import CustomRandint
from ..utils.dataset import get_samples_views_indices
from ..multiview_classifiers.spkm_pw import PWSPKM

classifier_class_name = "SampledPWSPKM"

class SampledPWSPKM(PWSPKM,):

    def __init__(self, random_state=42, n_u=2, kernel1=RBFKernel(0.5),
                 kernel2=RBFKernel(0.5), spkmregP=1, spkminit="randn",
                 nspkminits=10, preprocessinglist=[0,1,2], **kwargs):
        PWSPKM.__init__(self, random_state=random_state,
                                         n_u=n_u,
                                         kernel1=kernel1,
                                         kernel2=kernel2,
                                         spkmregP=spkmregP,
                                         spkminit=spkminit,
                                         nspkminits=nspkminits,
                                         preprocessinglist=preprocessinglist)
        self.rus = RandomUnderSampler(random_state=random_state)

    def fit(self, X, y, train_indices=None, view_indices=None):
        self.spkmregP=10
        self.lb =  LabelBinarizer(pos_label=1, neg_label=-1)
        y = self.lb.fit_transform(y)
        train_indices, view_indices = get_samples_views_indices(X,
                                                                 train_indices,
                                                                 view_indices)
        if len(view_indices)>2:
            self.more_than_two_views = True
            self.label_set = np.unique(y)
            return self
        self.used_views = view_indices
        self.view_names = [X.get_view_name(view_index)
                           for view_index in view_indices]
        view_list = [X.get_v(view_index)[train_indices, :]
                     for view_index in view_indices]
        self.rus.fit_resample(view_list[0], y[train_indices])
        resampled_list = [X[self.rus.sample_indices_, :] for X in view_list]
        return pairwiseSPKMlikeSklearn.fit(self, resampled_list, y[train_indices,0][self.rus.sample_indices_],)
