from .additions.early_fusion_from_monoview import BaseEarlyFusion
from ..utils.hyper_parameter_search import CustomRandint, CustomUniform
from ..monoview_classifiers.samba import SamBAClf
from SamBA.samba import *
from SamBA.distances import *
from SamBA.relevances import *
from sklearn.tree import DecisionTreeClassifier

classifier_class_name = "EarlyFusionSamba"


class EarlyFusionSamba(BaseEarlyFusion):

    def __init__(self, random_state=None, base_estimator=DecisionTreeClassifier(max_depth=1,
                                                       splitter='best',
                                                       criterion='gini'),
                 n_estimators=2,
                 estimator_params=tuple(),
                 relevance=ExpRelevance(),
                 distance=EuclidianDist(),
                 train_weighting=ExpTrainWeighting(),
                 keep_selected_features=True,
                 normalizer=RobustScaler(),
                 pred_train=False,
                 b=2, a=0.01,
                 forced_diversity=False,
                 normalize_dists=False,
                 **kwargs):
        BaseEarlyFusion.__init__(self, random_state=random_state,
                                 monoview_classifier="samba",
                                 base_estimator=base_estimator, estimator_params=estimator_params,
                                 relevance=relevance, distance=distance, train_weighting=train_weighting,
                                 keep_selected_features=keep_selected_features, normalizer=normalizer,
                                 n_estimators=n_estimators, pred_train=pred_train,
                                 b=b, a=a, forced_diversity=forced_diversity,
                                 normalize_dists=normalize_dists, **kwargs)
        self.param_names = ["n_estimators", "relevance", "distance",
                            "train_weighting", "b", "pred_train", "normalizer",
                            "normalize_dists", "a"]
        self.distribs = [CustomRandint(low=1, high=30),
                         [ExpRelevance()],
                         [EuclidianDist(), PolarDist(), ExpEuclidianDist()],
                         [ExpTrainWeighting()],
                         CustomUniform(0.2, 6),
                         [True, False],
                         [RobustScaler()],
                         [True], CustomRandint(0, 10, 'e-')]